<?php

namespace Controllers;

use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class LogociaController extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'logocia';
	protected $environment;
	protected $logociaService;

	public function __construct($crudService, ParameterBag $parameterBag, $environment, $logociaService){
		$this->environment = $environment;
		$this->entityManager = $entityManager;
		$this->logociaService = $logociaService;
		parent::__construct($crudService, $parameterBag);
	}

	public function upload($request, $response){
		$this->logociaService->atualizaBase($this->entityManager->getConnection());
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");

        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];
        $photo = empty($param['PHOTO_VIEW']) ? null : $param['PHOTO_VIEW'];
        $fileName = empty($param['FILE_NAME']) ? null : $param['FILE_NAME'];
        $nova = empty($param['NOVA']) ? null : $param['NOVA'];
        $cdcia = empty($param['CDCIA']) ? null : $param['CDCIA'];
        $sgcia = empty($param['SGCIA']) ? null : $param['SGCIA'];

        $extArq = substr($fileName, -3);

        if ($photo && $extArq && $cdcia && $sgcia){
	        try{
	            $path = $this->logociaService->uploadLogo($photo, $extArq, $cdcia, $sgcia);

	            $rowToSave = [array(
	            	"CDCIA" => $cdcia,
	            	"DSLOGOCIA" => $path,
	            	"DTULTATULOGO"=>$currentTime,
	            	"__is_new" => $nova
	            )];

				$newDataSet = new DataSet($this->dataSourceName, $rowToSave);
				$newMethod = $request->getMethod();
	            $newRoutePath =  $request->getRoutePath();
				$newUserId = $request->getUserId();

	            $newRequest = new DTO\Request\DataSet($newDataSet, $newMethod, $newRoutePath, $newUserId);

	            parent::save($newRequest, $response);
	        }catch (\Exception $e){
	        	$response->setError(new Error($e->getMessage(), 500));
	        }
	    } else {
	    		$response->setError(new Error("Ocorreu um erro ao enviar os parâmetros do arquivo para o servidor. Por favor tente novamente.", 500));
	    }
	}

	public function limpa($request, $response){
    $row = $request->getRow();
    $dslogocia = empty($row['DSLOGOCIA']) ? null : $row['DSLOGOCIA'];
		$this->logociaService->limpaLogo($dslogocia);
		parent::delete($request, $response);
	}
}