<?php

namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class SubstituicaoComposicaoController extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $environment;
    protected $connection;
    protected $newCode;
	protected $dataSourceName = "subscompmenu_escoposubscompmenu";
	protected $dataSourceNameToSave  = "subscompmenu";
	protected $escopoDataSourceName  = "escoposubscompmenu";
	protected $messagesProcedureInflightor;
	
	protected $tableName  = "SUBSCOMPMENU";
	
	const APROVA_PROCEDURE = 'APROVA_SUBSCOMPMENU';
	const GERAWORKFLOW_PROCEDURE = 'GERAWORKFLOW_SUBSCOMPMENU';
	
	const NRPROCESSOAPROVA_SUBSCOMPMENU_WF = 4;
	const NRITEMPROCAPROVA_SUBSCOMPMENU_WF = 27;
	
    const VALUE_TO_ALL = 'T';
    const FIELD_SEPARATOR = '-';
    const VALUE_SEPARATOR = '_';
    const IDSITUACAO_DESENVOLVIMENTO = 'E';
    
    const ESCOPO_FIELDS = array(
    	"CDFILIAL"	=> array(
    		"IDNIVELESCOPO" => 1,
    		"fields" => array("CDFILIAL")
    	),
    	"CIAFILI"	=> array(
    		"IDNIVELESCOPO" => 2,
    		"fields" => array("CDFILIAL", "CDCIA")
    	),
    	"MANCIA"	=> array(
    		"IDNIVELESCOPO" => 3,
    		"fields" => array("CDFILIAL", "CDCIA", "CDMANCIA")
    	),
    	"SERVCLASSE"=> array(
    		"IDNIVELESCOPO" => 4,
    		"fields" => array("CDFILIAL", "CDCIA", "CDMANCIA", "NRSERVICOCIAFILI", "CDCLASSE")
    	),
    	"SUBGRUPO"	=> array(
    		"IDNIVELESCOPO" => 5,
    		"fields" => array("CDFILIAL", "CDCIA", "CDMANCIA", "NRSERVICOCIAFILI", "CDCLASSE", "CDSUBGRU", "NRSEQSUBGRUSEMAM")
    	),
    	"CICLO" 	=> array(
    		"IDNIVELESCOPO" => 6,
    		"fields" => array("CDFILIAL", "CDCIA", "CDMANCIA", "NRSERVICOCIAFILI", "CDCLASSE", "CDSUBGRU", "NRSEQSUBGRUSEMAM", "CDCICLO")
    	)
    );

	public function __construct($crudService, $parameterBag, $environment, $connection, $newCode, $messagesProcedureInflightor) {
	    $this->environment = $environment;
	    $this->connection = $connection;
	    $this->newCode = $newCode;
		$this->messagesProcedureInflightor = $messagesProcedureInflightor;
		parent::__construct($crudService, $parameterBag);
	}

	public function saveSubscompmenu($request, $response){
		$dataSourceNameToSave = $this->dataSourceNameToSave;
		$connection = $this->connection;
		try {
			$connection->beginTransaction();
			$rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceNameToSave);
			$newRowsToSave = [];
			foreach ($rowsToSave as $rowToSave) {
				$dataEfetivacao = $rowToSave['DTEFETIVACAO'];
				if ($this->validateDtefetivacao($dataEfetivacao)) {
					$rowToSave['__escopoValues'] = array(
						"NRSUBSCOMPMENU" => $rowToSave["NRSUBSCOMPMENU"],
						"rows" => array()
					);
					$rowToSave['IDSITUACAO']		= $this->getIdDesenvolvimento();
					$rowToSave['TCDFILIAL'] 		= $this->treatEscopoValue($rowToSave, 'CDFILIAL');
					$rowToSave['TCIAFILI']			= $this->treatEscopoValue($rowToSave, 'CIAFILI');
					$rowToSave['TMANCIA']			= $this->treatEscopoValue($rowToSave, 'MANCIA');
					$rowToSave['TSERVCLASSE']		= $this->treatEscopoValue($rowToSave, 'SERVCLASSE');
					$rowToSave['TSUBGRUPO'] 		= $this->treatEscopoValue($rowToSave, 'SUBGRUPO');
					$rowToSave['TCICLO']			= $this->treatEscopoValue($rowToSave, 'CICLO');
					array_push($newRowsToSave, $rowToSave);
				} else {
					$errorMsg = 'COC03688_DT_EFET_INV';
					throw new \Exception($errorMsg);
				}
			}
			$this->crudService->save($dataSourceNameToSave, $newRowsToSave);
			$response->addDataset(new DataSource\DataSet('columnsToReturn', $this->crudService->getColumnsToReturn()));
			$escopoRowsToSave = [];
			foreach ($newRowsToSave as $savedRow) {
				$escopoRowsToSave = array_merge($escopoRowsToSave, $savedRow['__escopoValues']["rows"]);
			}
			if (sizeof($escopoRowsToSave) > 0) {
				$escopoDataSourceName = $this->escopoDataSourceName;
				
				$ecopoDataSet = new DataSource\DataSet($escopoDataSourceName, $escopoRowsToSave);
            	$ecopoRequest = new DTO\Request\DataSet($ecopoDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            	
            	$newEscopoRowsToSave = $this->crudService->getRowsToSave($ecopoRequest, $escopoDataSourceName);
				$this->crudService->save($escopoDataSourceName, $newEscopoRowsToSave);
			}
			$connection->commit();
		} catch (\Exception $e) {
			$connection->rollback();
			$response->setError(new Error($e->getMessage(), 1));
		}
	}
	
	public function deleteSubscompmenu($request, $response){
		$connection = $this->connection;
		
		$dataSourceName			= $this->dataSourceNameToSave;
		$escopoDataSourceName	= $this->escopoDataSourceName;
		
		$deleteEscopoQuery = 'DELETE FROM ESCOPOSUBSCOMPMENU WHERE NRSUBSCOMPMENU = :NRSUBSCOMPMENU';
		$deleteEditQuery = 'DELETE FROM EDITSUBSCOMPMENU WHERE NRSUBSCOMPMENU = :NRSUBSCOMPMENU';
		$deleteEscopoStmt = $connection->prepare($deleteEscopoQuery);
		$deleteEditStmt = $connection->prepare($deleteEditQuery);
		
		$request = $this->crudService->buildDataSetIfRow($request, $dataSourceName);
		$deleteDataset = $request->getDataSet();
		$rowsToDelete = $deleteDataset->getRows();
		try {
			$connection->beginTransaction();
			foreach ($rowsToDelete as $row) {
				$nrsubscompmenu = $row['NRSUBSCOMPMENU'];
				$nrorg = $this->environment->getNrOrgTrab();
				
				$this->validaDeleteSubscompmenu($nrsubscompmenu, $nrorg);
				
				$deleteParams = array(
					'NRSUBSCOMPMENU' => $nrsubscompmenu
				);
				$deleteEscopoStmt->execute($deleteParams);
				$deleteEditStmt->execute($deleteParams);
			}
			$this->crudService->delete($response, $deleteDataset);
			$connection->commit();
		} catch (\Exception $e) {
			$connection->rollback();
			$response->setError(new Error($e->getMessage(), 1));
		}
	}
	
	private function validaDeleteSubscompmenu($nrsubscompmenu, $nrorg) {
		$query = 'SELECT IDSITUACAO FROM SUBSCOMPMENU WHERE NRSUBSCOMPMENU = :NRSUBSCOMPMENU';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'NRSUBSCOMPMENU' => $nrsubscompmenu
		);
		
		$stmt->execute($params);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		if (sizeof($result) < 1) {
			$errorMsg = 'COC03688_LINHA_INV';
			throw new \Exception($errorMsg);
		} else {
			foreach($result as $row) {
				if ($row['IDSITUACAO'] != $this->getIdDesenvolvimento()) {
					$errorMsg = 'COC03688_SUBS_IN_PROP_APROV';
					throw new \Exception($errorMsg);
				}
			}
			return true;
		}
	}
	
	public function finalizaSubscompmenu($request, $response){
		$usaWorkflow = $this->verificaWorkflow();
		$acao = $usaWorkflow ? 'geraWorkflow' : 'aprovaSubscompmenu';
		
		$this->executaAcao($request, $response, $acao);
	}
	
	public function aprovaWorkflow($request, $response){
		$this->executaAcao($request, $response, 'aprovaSubscompmenu');
	}
	
	private function executaAcao($request, $response, $acao) {
		$nomeProcedure = '';
		$novoEstado = '';
		$mensagemRetorno = '';
		
		$connection = $this->connection;
		try {
			$connection->beginTransaction();
			
			$row = $this->getRowFromRequest($request);
			$this->validaRow($row);
			
			if ($acao == 'aprovaSubscompmenu') {
				$nomeProcedure = $this->getAprovaProcedure();
				$mensagemRetorno = 'COC03688_SUBS_APROV';
			} else if ($acao == 'geraWorkflow') {
				$nomeProcedure = $this->getGeraWorkflowProcedure();
				$mensagemRetorno = 'COC03688_WORKFLOW_GER';
			}
			
			$this->chamaProcedure($row, $response, $nomeProcedure);
			$response->addMessage(new Message($mensagemRetorno));
			
			$connection->commit();
		} catch (\Exception $e) {
			$connection->rollback();
			$response->setError(new Error($e->getMessage(), 1));
		}
	}
	
	private function chamaProcedure($row, $response, $nomeProcedure) {
		$nrsubscompmenu = $row['NRSUBSCOMPMENU'];
		$filialLogada = $this->environment->getCdFilial();
		$nrorg = $this->environment->getNrOrgTrab();
		$cdoperador = $this->environment->getCdOperador();
		
        $procedure = new StoredProcedure($this->connection, $nomeProcedure);
        
        $procedure->addParam(new Param('P_NRSUBSCOMPMENU', Param::PARAM_INPUT, $nrsubscompmenu, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filialLogada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
	}
	
	private function verificaWorkflow() {
		$query = 'SELECT IDATIVO FROM ITEMPROCAPROVA WHERE NRPROCESSOAPROVA = :NRPROCESSOAPROVA AND NRITEMPROCAPROVA = :NRITEMPROCAPROVA';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'NRPROCESSOAPROVA' => $this->getNrProcessoaprova(),
			'NRITEMPROCAPROVA' => $this->getNrItemProcaprova()
		);
		
		$stmt->execute($params);
		
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		if (sizeof($result) < 1) {
			return false;
		} else {
			foreach($result as $row) {
				if ($row['IDATIVO'] == 'S') {
					return true;
				}
			}
			return false;
		}
	}
	
	private function getRowFromRequest($request) {
		$row = array();
		if (method_exists($request , 'getRow')) {
			$row = $request->getRow();
		} else if (method_exists($request , 'getDataSet')) {
			$dataSet = $request->getDataSet();
			if (method_exists($dataSet, 'getRows')) {
				$rows = $dataSet->getRows();
				if (sizeof($rows) > 0) {
					$row = $rows[0];
				}
			}
		}
		return $row;
	}
	
	private function validaRow($row) {
		if ($row && gettype($row) == 'object') {
			if (!$row['NRSUBSCOMPMENU']) {
				$errorMsg = 'COC03688_CHAVE_NOT_INF';
				throw new \Exception($errorMsg);
			}
		} else {
			$errorMsg = 'COC03688_LINHA_INV';
			throw new \Exception($errorMsg);
		}
	}
	
	private function validateDtefetivacao($dataStr) {
		$data = date_create_from_format('d/m/Y', $dataStr);
		return $data > date('d/m/Y');
	}
	
	private function treatEscopoValue(&$rowToSave, $fieldToTreat) {
		$valueToAll = $this->getValueToAll();
		$value = $rowToSave[$fieldToTreat];
		
		if ($value) {
			if ($value == $valueToAll) {
				return $valueToAll;
			} else {
				$this->pushToEscopoRows($rowToSave, $fieldToTreat, $value);
				return null;
			}
		} else {
			$errorMsg = 'COC03688_VALOR_CAMPO_N_ENCONTRADO' . '#_#' .  $fieldToTreat;
			throw new \Exception($errorMsg);
		}
	}
	
	private function pushToEscopoRows(&$rowToSave, $fieldToTreat, $fieldValue) {
		$valueSeparator = $this->getValueSeparator();
		if(is_array($fieldValue)){
			$valueArray = $fieldValue;
		} else {
			$valueArray = explode($valueSeparator, $fieldValue);
		}
		
		foreach ($valueArray as $value) {
			$escopoRowToSave = $this->createEscopoRowToSave($fieldToTreat, $value);
			$escopoRowToSave['NRSUBSCOMPMENU'] = $rowToSave['__escopoValues']['NRSUBSCOMPMENU'];
			array_push($rowToSave['__escopoValues']['rows'], $escopoRowToSave);
		}
		
		return $rowToSave;
	}
	
	private function createEscopoRowToSave($fieldToTreat, $fieldValue){
		$fieldSeparator 	= $this->getFieldSeparator();
		$escopoFields		= $this->getEscopoFields();
		$escopoField		= $escopoFields[$fieldToTreat];
		$escopoFieldArray	= $escopoField['fields'];
		
		$fieldsValueArray = explode($fieldSeparator, $fieldValue);
		
		$escopoRow = array(
			"__is_new" => true,
			"__new_code_pk" => "ESCOPOSUBSCOMPMENU",
			"__table_pk" => "NRESCOPOSUBSCM",
			"__new_code_length" => 5,
			"__cast_number" => true,
			"IDNIVELESCOPO" => $escopoField["IDNIVELESCOPO"]
		);
		
		if ($escopoFieldArray && (sizeof($fieldsValueArray) == sizeof($escopoFieldArray))) {
			for ($i = 0; $i < sizeof($fieldsValueArray); $i++) {
				$valueToSave		= $fieldsValueArray[$i];
				$escopoFieldToSave	= $escopoFieldArray[$i];
				
				$escopoRow[$escopoFieldToSave] = $valueToSave;
			}
			return $escopoRow;
		} else {
			$errorMsg = 'COC03688_REGRA_CAMPO_N_ENCONTRADO' . '#_#' .  $fieldToTreat;
			throw new \Exception($errorMsg);
		}
	}
	
	private function getAprovaProcedure() {
		return SELF::APROVA_PROCEDURE;
	}
	
	private function getGeraWorkflowProcedure() {
		return SELF::GERAWORKFLOW_PROCEDURE;
	}
	
	private function getNrProcessoaprova() {
		return SELF::NRPROCESSOAPROVA_SUBSCOMPMENU_WF;
	}
	private function getNrItemProcaprova() {
		return SELF::NRITEMPROCAPROVA_SUBSCOMPMENU_WF;
	}
	
	private function getValueToAll() {
		return SELF::VALUE_TO_ALL;
	}
	
	private function getFieldSeparator() {
		return SELF::FIELD_SEPARATOR;
	}
	
	private function getValueSeparator() {
		return SELF::VALUE_SEPARATOR;
	}
	
	private function getEscopoFields() {
		return SELF::ESCOPO_FIELDS;
	}
	
	private function getIdDesenvolvimento() {
		return SELF::IDSITUACAO_DESENVOLVIMENTO;
	}
}